(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 4.2'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     37939,       1716]*)
(*NotebookOutlinePosition[     39319,       1759]*)
(*  CellTagsIndexPosition[     39234,       1753]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Generating symbolic expressions by templates", "Title",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Christian Jacob, jacob@cpsc.ucalgary.ca  April 1997 / February 2001 \
\[Copyright]\
\>", "Subsubtitle"],

Cell["\<\
{
  {f[BlankSequence[_h | _g | _f]], 2},
  {f[_h | _g],                     4},
  
  {g[a | b | c],                   1},
  {g[_Integer, __Symbol],          2},
  
  {h[_Integer | _Symbol],          1},
  
  {a, 1}, {b, 1}, {c, 1}, {d, 1}, {e, 1},
  
  Sequence @@ Table[{i,1},{i,1,5}]
  
}\
\>", "InputDemo",
  AspectRatioFixed->True,
  CellTags->"Weighted templates"],

Cell[CellGroupData[{

Cell["Introduction", "Subtitle",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "This notebook demonstrates the basic idea of how to generate \
problem-specific symbolic expressions by using ",
  StyleBox["templates",
    FontWeight->"Bold"],
  " as building blocks."
}], "Text"],

Cell[TextData[{
  "In the most simple case, templates are just ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " patterns. In order to specify more complex building block structures, we \
introduce patterns with ",
  StyleBox["alternatives",
    FontSlant->"Italic"],
  " as well as ",
  StyleBox["weight",
    FontSlant->"Italic"],
  "-",
  StyleBox["attributed",
    FontSlant->"Italic"],
  " templates. "
}], "Text"],

Cell["\<\
The functions discussed in this notebook are used for generating \
problemspecific data structures or computer programs. Furthermore, the \
mutation operators on symbolic expressions (see Chapters 7 and 8) use the \
same mechanisms to generate new substructures.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subtitle",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Please make sure to evaluate all ",
  StyleBox["Initialization Cells",
    FontSlant->"Italic"],
  " in this notebook, ",
  StyleBox["before",
    FontWeight->"Bold"],
  " you try any of the following examples."
}], "Text"],

Cell[CellGroupData[{

Cell["Simple expression templates: example 1", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Templates", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  g[a], g[b], g[c],
  
  h[_Integer], h[_Symbol],
  
  d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[5, f[_g, _, _, _h],
  ExprPatterns -> patterns] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[a], 5, g[c], h[e]]\)},
          {\(f[g[a], h[1], 4, h[4]]\)},
          {\(f[g[b], h[d], 1, h[e]]\)},
          {\(f[g[c], 3, g[b], h[4]]\)},
          {\(f[g[c], 4, h[5], h[d]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ a], 5, 
          g[ c], 
          h[ e]], 
        f[ 
          g[ a], 
          h[ 1], 4, 
          h[ 4]], 
        f[ 
          g[ b], 
          h[ d], 1, 
          h[ e]], 
        f[ 
          g[ c], 3, 
          g[ b], 
          h[ 4]], 
        f[ 
          g[ c], 4, 
          h[ 5], 
          h[ d]]}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Simple expression templates: example 2", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Templates", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  f[_g, _, _, _h],
  
  g[a], g[b], g[c],
  
  h[_Integer], h[_Symbol],
  
  d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[10, _f,
  ExprPatterns -> patterns] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[a], e, g[a], h[3]]\)},
          {\(f[g[a], e, h[5], h[4]]\)},
          {\(f[g[a], g[b], h[e], h[e]]\)},
          {\(f[g[a], h[4], h[e], h[d]]\)},
          {\(f[g[a], h[5], h[4], h[e]]\)},
          {\(f[g[b], 1, e, h[2]]\)},
          {\(f[g[b], 3, e, h[e]]\)},
          {\(f[g[b], h[2], g[c], h[d]]\)},
          {\(f[g[c], g[c], 3, h[d]]\)},
          {\(f[g[c], h[1], 3, h[d]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ a], e, 
          g[ a], 
          h[ 3]], 
        f[ 
          g[ a], e, 
          h[ 5], 
          h[ 4]], 
        f[ 
          g[ a], 
          g[ b], 
          h[ e], 
          h[ e]], 
        f[ 
          g[ a], 
          h[ 4], 
          h[ e], 
          h[ d]], 
        f[ 
          g[ a], 
          h[ 5], 
          h[ 4], 
          h[ e]], 
        f[ 
          g[ b], 1, e, 
          h[ 2]], 
        f[ 
          g[ b], 3, e, 
          h[ e]], 
        f[ 
          g[ b], 
          h[ 2], 
          g[ c], 
          h[ d]], 
        f[ 
          g[ c], 
          g[ c], 3, 
          h[ d]], 
        f[ 
          g[ c], 
          h[ 1], 3, 
          h[ d]]}]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
createExpression[_f, MaxDepth -> 2, ExprPatterns -> patterns]\
\>", \
"Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[\("Warning: No pattern free expression after "\
\[InvisibleSpace]10\[InvisibleSpace]" trials."\),
      SequenceForm[ 
      "Warning: No pattern free expression after ", 10, " trials."],
      Editable->False]], "Print"],

Cell[BoxData[
    \(f[g[c], h[_Integer], h[_Integer], h[_Symbol]]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
createExpression[_f, MaxDepth -> 3, ExprPatterns -> patterns]\
\>", \
"Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(f[g[b], 2, d, h[3]]\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Variable number of arguments", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Templates (1)", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  f[_g, __, _h],
  
  g[a], g[b], g[c],
  
  h[_Integer], h[_Symbol],
  
  d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Example (1)", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[3, _f,
                 SequenceRange -> {1,5},
                 ExprPatterns :> patterns] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[a], 2, 5, h[d]]\)},
          {\(f[g[a], 3, h[4], h[e], h[3]]\)},
          {\(f[g[b], g[c], 5, 1, h[1]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ a], 2, 5, 
          h[ d]], 
        f[ 
          g[ a], 3, 
          h[ 4], 
          h[ e], 
          h[ 3]], 
        f[ 
          g[ b], 
          g[ c], 5, 1, 
          h[ 1]]}]]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Templates (2)", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  f[_g, ___, _h],
  
  g[a], g[b], g[c],
  
  h[_Integer], h[_Symbol],
  
  d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Example (2)", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[3, _f,
                 SequenceRange -> {1,5},
                 ExprPatterns :> patterns] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[a], h[d]]\)},
          {\(f[g[b], h[d]]\)},
          {\(f[g[a], g[a], h[4], g[a], e, g[c], h[4]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ a], 
          h[ d]], 
        f[ 
          g[ b], 
          h[ d]], 
        f[ 
          g[ a], 
          g[ a], 
          h[ 4], 
          g[ a], e, 
          g[ c], 
          h[ 4]]}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Variable number of arguments (2)", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Templates", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  f[Sequence @@ Table[_,{Random[Integer,{1,5}]}]],
  
  g[a], g[b], g[c],
  
  h[_Integer], h[_Symbol],
  
  d, e
};

terminals = Range[1,5];\
\>", "Input",
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpression[_f,
                 MaxDepth -> 6,
                 ExprPatterns :> patterns,
                 ExprTerminals :> terminals]\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(f[g[b], g[a], e, g[a]]\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Alternative arguments", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Templates", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  f[_g | _h, ___, _g],
  
  g[a | b | c],
  
  h[_Integer | _Symbol],
  
  d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[10, _f,
                 SequenceRange -> {1,5},
                 ExprPatterns :> patterns] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[{}], g[{}]]\)},
          {\(f[h[5], g[{}]]\)},
          {\(f[g[{}], 2, g[{}]]\)},
          {\(f[h[1], 3, g[{}]]\)},
          {\(f[h[3], 
              f[h[1], f[g[{}], g[{}], g[{}]], 
                f[g[{}], e, d, 5, d, g[{}], g[{}]], e, 
                f[g[{}], g[{}], 1, 
                  f[g[{}], h[5], f[h[5], 3, g[{}]], d, h[4], 2, g[{}]], 4, 4, 
                  g[{}]], 2, g[{}]], g[{}]]\)},
          {\(f[h[5], 2, g[{}]]\)},
          {\(f[g[{}], f[g[{}], f[g[{}], 1, g[{}]], g[{}]], h[e], 4, e, 2, 
              g[{}]]\)},
          {\(f[g[{}], g[{}], 2, f[g[{}], h[d], g[{}]], d, 3, g[{}]]\)},
          {\(f[h[3], h[3], f[h[d], g[{}]], 2, 1, e, g[{}]]\)},
          {\(f[h[3], h[d], 4, d, f[h[e], 4, g[{}]], 4, g[{}]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ {}], 
          g[ {}]], 
        f[ 
          h[ 5], 
          g[ {}]], 
        f[ 
          g[ {}], 2, 
          g[ {}]], 
        f[ 
          h[ 1], 3, 
          g[ {}]], 
        f[ 
          h[ 3], 
          f[ 
            h[ 1], 
            f[ 
              g[ {}], 
              g[ {}], 
              g[ {}]], 
            f[ 
              g[ {}], e, d, 5, d, 
              g[ {}], 
              g[ {}]], e, 
            f[ 
              g[ {}], 
              g[ {}], 1, 
              f[ 
                g[ {}], 
                h[ 5], 
                f[ 
                  h[ 5], 3, 
                  g[ {}]], d, 
                h[ 4], 2, 
                g[ {}]], 4, 4, 
              g[ {}]], 2, 
            g[ {}]], 
          g[ {}]], 
        f[ 
          h[ 5], 2, 
          g[ {}]], 
        f[ 
          g[ {}], 
          f[ 
            g[ {}], 
            f[ 
              g[ {}], 1, 
              g[ {}]], 
            g[ {}]], 
          h[ e], 4, e, 2, 
          g[ {}]], 
        f[ 
          g[ {}], 
          g[ {}], 2, 
          f[ 
            g[ {}], 
            h[ d], 
            g[ {}]], d, 3, 
          g[ {}]], 
        f[ 
          h[ 3], 
          h[ 3], 
          f[ 
            h[ d], 
            g[ {}]], 2, 1, e, 
          g[ {}]], 
        f[ 
          h[ 3], 
          h[ d], 4, d, 
          f[ 
            h[ e], 4, 
            g[ {}]], 4, 
          g[ {}]]}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Alternative variable arguments", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Templates", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  f[BlankSequence[_h | (g[x__]/; Length[{x}] > 1)]],
  
  g[a | b | c],
  g[_Integer, __Symbol],
  
  h[_Integer | _Symbol],
  
  a, b, c, d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[5, _f,
                 SequenceRange -> {1,5},
                 ExprPatterns :> patterns] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[4, d, c, b, d, c]]\)},
          {\(f[h[c], g[5, a, b, c]]\)},
          {\(f[g[2, d, d, e, c], g[4, c, e, b, b], g[2, d, a, d]]\)},
          {\(f[h[b], h[3], g[4, c, b, c, e], h[4]]\)},
          {\(f[h[d], h[c], g[1, d, c, a, e, e], h[c], h[1]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ 4, d, c, b, d, c]], 
        f[ 
          h[ c], 
          g[ 5, a, b, c]], 
        f[ 
          g[ 2, d, d, e, c], 
          g[ 4, c, e, b, b], 
          g[ 2, d, a, d]], 
        f[ 
          h[ b], 
          h[ 3], 
          g[ 4, c, b, c, e], 
          h[ 4]], 
        f[ 
          h[ d], 
          h[ c], 
          g[ 1, d, c, a, e, e], 
          h[ c], 
          h[ 1]]}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["S-expressions versus terms", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["S-expression templates", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  s[_g, _Integer][_g | _h, ___, _Integer | _Symbol],
  
  g[a | b | c],
  
  h[_Integer | _Symbol],
  
  a, b, c, d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[5, s[_h, ___Integer][__g, _Symbol],
                 SequenceRange -> {2,4},
                 ExprPatterns :> patterns] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(\(s[h[a], 1, 3]\)[g[c], g[a], g[b], g[a], e]\)},
          {\(\(s[h[d], 4, 1]\)[g[b], g[b], g[c], c]\)},
          {\(\(s[h[2], 4, 5, 1, 5]\)[g[c], g[a], d]\)},
          {\(\(s[h[4], 1, 3, 5, 2]\)[g[c], g[a], g[a], g[c], a]\)},
          {\(\(s[h[c], 3, 4, 5, 4]\)[g[a], g[c], c]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        s[ 
          h[ a], 1, 3][ 
          g[ c], 
          g[ a], 
          g[ b], 
          g[ a], e], 
        s[ 
          h[ d], 4, 1][ 
          g[ b], 
          g[ b], 
          g[ c], c], 
        s[ 
          h[ 2], 4, 5, 1, 5][ 
          g[ c], 
          g[ a], d], 
        s[ 
          h[ 4], 1, 3, 5, 2][ 
          g[ c], 
          g[ a], 
          g[ a], 
          g[ c], a], 
        s[ 
          h[ c], 3, 4, 5, 4][ 
          g[ a], 
          g[ c], c]}]]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Weight-attributed templates", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Example 1: Templates and weights separated", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns :=
{
  f[BlankSequence[_h | _g | _f]],
  f[_h | _g],
  
  g[a | b | c],
  g[_Integer, __Symbol],
  
  h[_Integer | _Symbol],
  
  a, b, c, d, e,
  
  Sequence @@ Range[1,5]
}\
\>", "Input",
  AspectRatioFixed->True],

Cell["\<\
weights =
{
  2,
  4,
  
  1,
  2,
  
  1,
  
  1, 1, 1, 1, 1,
  1, 1, 1, 1, 1
};\
\>", "Input",
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[10, _f,
                 SequenceRange -> {3,5},
                 ExprPatterns :> patterns,
                 ExprWeights  -> weights  ] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[1, e, a, c]]\)},
          {\(f[g[2, c, b, c]]\)},
          {\(f[g[4, d, a, e, d]]\)},
          {\(f[g[3, a, a, e, e, e]]\)},
          {\(f[g[3, c, d, e, b, e]]\)},
          {\(f[h[5]]\)},
          {\(f[h[b]]\)},
          {\(f[g[1, b, c, d, e], f[g[4, b, e, c]], g[1, c, d, b, d], 
              f[h[e], f[g[c]], g[a], g[3, a, c, c]]]\)},
          {\(f[g[1, a, c, e, e], g[b], g[3, a, a, a, e, b], g[2, b, d, c, d], 
              g[c]]\)},
          {\(f[h[2], f[g[5, d, d, a, d]], g[1, d, e, c, a], 
              f[f[g[4, e, e, a]], 
                f[g[5, c, e, c], 
                  f[g[a], f[g[5, a, e, b, b]], 
                    f[g[c], g[2, d, e, d, a], g[5, b, a, b, e, d]]], h[3], 
                  f[h[4]], g[2, c, d, b]], f[g[3, c, c, e, b, e]]], 
              f[g[2, c, d, e]]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ 1, e, a, c]], 
        f[ 
          g[ 2, c, b, c]], 
        f[ 
          g[ 4, d, a, e, d]], 
        f[ 
          g[ 3, a, a, e, e, e]], 
        f[ 
          g[ 3, c, d, e, b, e]], 
        f[ 
          h[ 5]], 
        f[ 
          h[ b]], 
        f[ 
          g[ 1, b, c, d, e], 
          f[ 
            g[ 4, b, e, c]], 
          g[ 1, c, d, b, d], 
          f[ 
            h[ e], 
            f[ 
              g[ c]], 
            g[ a], 
            g[ 3, a, c, c]]], 
        f[ 
          g[ 1, a, c, e, e], 
          g[ b], 
          g[ 3, a, a, a, e, b], 
          g[ 2, b, d, c, d], 
          g[ c]], 
        f[ 
          h[ 2], 
          f[ 
            g[ 5, d, d, a, d]], 
          g[ 1, d, e, c, a], 
          f[ 
            f[ 
              g[ 4, e, e, a]], 
            f[ 
              g[ 5, c, e, c], 
              f[ 
                g[ a], 
                f[ 
                  g[ 5, a, e, b, b]], 
                f[ 
                  g[ c], 
                  g[ 2, d, e, d, a], 
                  g[ 5, b, a, b, e, d]]], 
              h[ 3], 
              f[ 
                h[ 4]], 
              g[ 2, c, d, b]], 
            f[ 
              g[ 3, c, c, e, b, e]]], 
          f[ 
            g[ 2, c, d, e]]]}]]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Example  2: Templates and weights combined", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patternsAndWeights =
{
  {f[BlankSequence[_h | _g | _f]], 2},
  {f[_h | _g],                     4},
  
  {g[a | b | c],                   1},
  {g[_Integer, __Symbol],          2},
  
  {h[_Integer | _Symbol],          1},
  
  {a, 1}, {b, 1}, {c, 1}, {d, 1}, {e, 1},
  
  Sequence @@ Table[{i,1},{i,1,5}]
  
} //Transpose;\
\>", "Input",
  AspectRatioFixed->True,
  CellTags->"Weighted templates"],

Cell[CellGroupData[{

Cell["patternsAndWeights", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \({{f[BlankSequence[_h | _g | _f]], f[_h | _g], g[a | b | c], 
        g[_Integer, __Symbol], h[_Integer | _Symbol], a, b, c, d, e, 1, 2, 3, 
        4, 5}, {2, 4, 1, 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
createExpressions[10, _f,
  SequenceRange -> {3,5},
  ExprPatterns -> patternsAndWeights[[1]],
  ExprWeights  -> patternsAndWeights[[2]]  ] // TableForm\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(f[g[c]]\)},
          {\(f[g[2, c, e, b]]\)},
          {\(f[g[5, a, e, d]]\)},
          {\(f[g[1, a, b, c, e]]\)},
          {\(f[g[4, a, e, e, a, e]]\)},
          {\(f[g[4, e, b, e, b, a]]\)},
          {\(f[f[g[b]], g[4, c, c, e], g[4, d, a, c, a, c], f[h[d]]]\)},
          {\(f[g[c], f[h[a]], g[5, b, c, e, a, c], h[4]]\)},
          {\(f[f[h[5]], f[h[d]], 
              f[f[g[3, a, c, e, a, d]], g[4, c, b, e, c, c], 
                f[g[2, c, c, b, b, a]], f[g[c]], g[b]], f[h[1]], g[c]]\)},
          {\(f[
              f[f[h[3]], g[a], f[g[1, c, e, a, a, e]], g[2, b, d, b], 
                f[h[5]]], f[h[2]], f[g[1, e, c, a]], g[4, e, a, c, d, e], 
              f[g[2, b, a, a]]]\)}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      TableForm[ {
        f[ 
          g[ c]], 
        f[ 
          g[ 2, c, e, b]], 
        f[ 
          g[ 5, a, e, d]], 
        f[ 
          g[ 1, a, b, c, e]], 
        f[ 
          g[ 4, a, e, e, a, e]], 
        f[ 
          g[ 4, e, b, e, b, a]], 
        f[ 
          f[ 
            g[ b]], 
          g[ 4, c, c, e], 
          g[ 4, d, a, c, a, c], 
          f[ 
            h[ d]]], 
        f[ 
          g[ c], 
          f[ 
            h[ a]], 
          g[ 5, b, c, e, a, c], 
          h[ 4]], 
        f[ 
          f[ 
            h[ 5]], 
          f[ 
            h[ d]], 
          f[ 
            f[ 
              g[ 3, a, c, e, a, d]], 
            g[ 4, c, b, e, c, c], 
            f[ 
              g[ 2, c, c, b, b, a]], 
            f[ 
              g[ c]], 
            g[ b]], 
          f[ 
            h[ 1]], 
          g[ c]], 
        f[ 
          f[ 
            f[ 
              h[ 3]], 
            g[ a], 
            f[ 
              g[ 1, c, e, a, a, e]], 
            g[ 2, b, d, b], 
            f[ 
              h[ 5]]], 
          f[ 
            h[ 2]], 
          f[ 
            g[ 1, e, c, a]], 
          g[ 4, e, a, c, d, e], 
          f[ 
            g[ 2, b, a, a]]]}]]], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["An L-system example", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
The following templates are used in Chapter 11, where we evolve \
rule systems (in the form of Lindenmayer systems), that encode growth \
programs for plant-like, branching structures.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Templates describing rule systems", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
patterns =
{
  LSystem[_AXIOM, _LRULES],
  
  AXIOM[sprout[4]],
  
  LRULES[
    LSysRule[LEFT[], PRED[ sprout[4] ], RIGHT[],
      SUCC[SEQ[f],
           __SEQ,
           bloom[0]]],
    __LSysRule
  ],
  
  LSysRule[LEFT[], _PRED, RIGHT[], _SUCC],
  
  PRED[ sprout[aIndex]],
  
  SUCC[_SEQ | _STACK],
  
  SEQ[BlankSequence[_sprout | _stalk | _leaf | _SEQ]],
  SEQ[BlankSequence[_sprout | _stalk | _leaf ]],
  
  STACK[BlankSequence[_sprout | _stalk | _leaf | _STACK]],
  STACK[BlankSequence[_sprout | _stalk | _leaf ]]
};

terminals =
{
  sprout[sproutIndex], stalk[stalkIndex], leaf[leafIndex]
};\
\>", "Input",
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpression[_LSystem,
                 ArgumentRange -> {1,5},
                 MaxDepth -> 30,
                 ExprPatterns -> patterns,
                 ExprTerminals -> terminals]\
\>", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(LSystem[AXIOM[sprout[4]], 
      LRULES[LSysRule[LEFT[], PRED[sprout[4]], RIGHT[], 
          SUCC[SEQ[f], SEQ[SEQ[SEQ[SEQ[SEQ[{}]]]]], bloom[0]]], 
        LSysRule[LEFT[], PRED[sprout[aIndex]], RIGHT[], 
          SUCC[SEQ[{}]]]]]\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementation", "Subtitle",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Generating Expressions", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Generating Lists of Expressions", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpressions[howMany_Integer, startPattern_, opts___] :=
Module[{exprs = {}, lim = 10 howMany, i=0},

  While[(Length[exprs] < howMany) && (i < lim),
  
    exprs = Union[exprs ~Join~ 
      {createExpression[startPattern, opts]}];
    i++
  ];
  
  If[i === lim, 
    Print[\"Warning: \",howMany,
          \" diff. expressions not possible.\"]];
    
  exprs
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(createExpression\)\" is \
similar to existing symbol \"\!\(createExpressions\)\"."\)], "Message"]
}, Open  ]],

Cell["Options[createExpressions] := Options[createExpression]", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["createExpressions[2, x, ExprPatterns -> {a}]", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    InterpretationBox[\("Warning: "\[InvisibleSpace]2\[InvisibleSpace]" diff. \
expressions not possible."\),
      SequenceForm[ "Warning: ", 2, " diff. expressions not possible."],
      Editable->False]], "Print"],

Cell[BoxData[
    \({x}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generation Function for Expressions", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
createExpression[startPattern_, opts___] :=
Module[{patts, depth, cont = True, i = 1, tr, expr},

  depth = MaxDepth /. {opts} /. Options[createExpression];
  patts = ExprPatterns /. {opts} /. Options[createExpression];
  tr    = Trials /. {opts} /. Options[createExpression];
  
  While[cont,
    expr = randomExpr[startPattern, depth, patts, opts];
             
    If[Not[PatternQ[expr]], cont = False];
    
    If[i > tr,
      Print[\"Warning: No pattern free expression after \",
        tr,\" trials.\"];
      cont = False
    ];
    i++
  ];
  
  expr
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(depth\)\" is similar to \
existing symbol \"\!\(Depth\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(expr\)\" is similar to \
existing symbol \"\!\(exprs\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(PatternQ\)\" is similar \
to existing symbol \"\!\(Pattern\)\"."\)], "Message"],

Cell[BoxData[
    \(General::"stop" \(\(:\)\(\ \)\) 
      "Further output of \!\(General :: \"spell1\"\) will be suppressed \
during this calculation."\)], "Message"]
}, Open  ]],

Cell["\<\
Options[createExpression] =
{
  MaxDepth -> 10, Trials -> 10,
  ExprPatterns -> {}, ExprWeights -> {}
};\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Random Expressions: randomExpr", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["ClearAll[randomExpr]", "Input",
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Variable Number of Arguments", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
randomExpr[pat_BlankSequence, depth_?Positive,
           patterns_List, opts___] :=
Module[{argRange},
  
  argRange = SequenceRange /. {opts} /. Options[randomExpr];
  
  Sequence @@ Table[
    randomExpr[
      If[Length[pat]>0,
        If[Head[First[pat]] === Alternatives,
          First[pat],
          Blank[First[pat]]],
        _],
      depth, patterns, opts],
    {Random[Integer,argRange]}]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
randomExpr[pat_BlankNullSequence, depth_?Positive,
           patterns_List, opts___] :=
Module[{argRange},
  
  argRange = SequenceRange /. {opts} /. Options[randomExpr];
  
  Sequence @@ Table[
    randomExpr[
      If[Length[pat]>0,
        If[Head[First[pat]] === Alternatives,
          First[pat],
          Blank[First[pat]]],
        _],
      depth, patterns, opts],
    {randomSelect[Union[{0},argRange]]}]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Alternative Arguments", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
randomExpr[pat_Alternatives, depth_?Positive,
           patterns_List, opts___] :=
Module[{},
  
  (*randomExpr[randomSelect[pat], depth, patterns, opts]*)
  
  randomExpr[
    randomSelectWeight[patterns,pat,opts], 
    depth, patterns, opts
  ]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Condition", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
randomExpr[pat_Condition, depth_?Positive,
           patterns_List, opts___] :=
Module[{randPat},
  
  randPat = randomSelectWeight[patterns, pat, opts];
  
  If[randPat === {},
    pat,
    randomExpr[randPat, depth, patterns, opts]
  ]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Blank Expressions", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
randomExpr[pat_Blank, depth_?Positive,
           patterns_List, opts___] :=
Module[{randPat},
  
  randPat = randomSelectWeight[patterns, pat, opts];
  
  If[randPat === {},
    pat,
    randomExpr[randPat, depth, patterns, opts]
  ]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Expressions with Patterns or Alternatives", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
randomExpr[startPattern_/; (PatternQ[startPattern] ||
                            AlternativesQ[startPattern]), 
           maxDepth_?Positive,
           patterns_List, opts___] :=
Module[{},
      
  Map[
    randomExpr[#, maxDepth-1,patterns,opts]&, 
    startPattern, Heads -> True]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(AlternativesQ\)\" is \
similar to existing symbol \"\!\(Alternatives\)\"."\)], "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Expressions without Patterns or Alternatives", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
randomExpr[np_/; Not[PatternQ[np]] && Not[AlternativesQ[np]], 
           maxDepth_?NonNegative,
           patterns_List, opts___] := np\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Level-0 Expressions", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["randomExpr[pat_, 0, _List, opts___] := pat", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Options", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Options[randomExpr] =
{
  SequenceRange -> {1,1}
};\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Utility Functions", "Section",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Random Selection: randomSelect, randomSelectWeight, fitPropSelect\
\
\>", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Random Selection", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["randomSelect[s_] := s[[Random[Integer,{1,Length[s]}]]]", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["randomSelect[{}] = {};", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Selection of Weighted Patterns", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
randomSelectWeight[selExpr_, pat_, opts___] :=
Module[{weights, matchPat, matchPos, matchWeights},

  weights = ExprWeights /. {opts} 
            /. {ExprWeights :> Table[1,{Length[selExpr]}]};
  
  {matchPat, matchPos} = 
    {Cases[selExpr, pat], 
     Position[selExpr, pat, 1, Heads -> False]};
  (**Print[\"matchPat: \",matchPat];**)
  (**Print[\"matchPos: \",matchPos];**)
  
  matchWeights = Map[weights[[#//First]]&, matchPos];
  (**Print[\"matchWeights: \", matchWeights];**)
  pos = fitPropSelect[matchWeights];
  (**Print[\"pos: \",pos];**)
  matchPat[[pos]]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Fitness-proportionate Selection", "Subsubsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
fitPropSelect[fitnesses_, opts___] :=
Module[{cumFits, fits, r, m},

  fits = 
  If[Negative[m = Min[fitnesses]], fitnesses -m, fitnesses];
  
  cumFits = FoldList[Plus, 0, fits] // Rest;
  
  r = Random[] Last[cumFits];
  
  Position[cumFits, _?(# >= r &), {1}, 1][[1,1]]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["fitPropSelect[{}, opts___] := {}", "Input",
  InitializationCell->True,
  AspectRatioFixed->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Testing for Pattern Expressions: PatternQ", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
PatternQ[x_] :=
Module[{realPats,pats},

  {realPats,pats} = {{Blank, BlankSequence,BlankNullSequence},
                     {BLANK, BLANKSEQ, BLANKNULLSEQ}};

  If[Count[{x} /. MapThread[#1->#2&,{realPats,pats}],
       Alternatives @@ Map[#[___]&,pats],Infinity,
       Heads -> True] > 0, 
    True, False
  ]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(General::"spell" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(pats\)\" is similar to \
existing symbols \!\({pat, patts}\)."\)], "Message"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Testing for Alternatives: AlternativesQ", "Subsection",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
AlternativesQ[x_] :=
Module[{realPats,pats},

  {realPats,pats} = {{Alternatives},
                     {ALTERNATIVES}};

  If[Count[{x} /. MapThread[#1->#2&,{realPats,pats}],
       Alternatives @@ Map[#[___]&,pats],Infinity,
       Heads -> True] > 0, 
    True, False
  ]
]\
\>", "Input",
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["AlternativesQ[g[a ]]", "Input",
  AspectRatioFixed->True],

Cell[BoxData[
    \(False\)], "Output"]
}, Open  ]]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["References", "Subtitle",
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "C. Jacob, ",
  StyleBox["Illustrating Evolutionary Computation with Mathematica.",
    FontWeight->"Bold"],
  " Morgan Kaufmann Publishers, San Fancisco, 2001. Chapter 8.\n"
}], "Text"]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"4.2 for Macintosh",
ScreenRectangle->{{0, 1152}, {0, 746}},
AutoGeneratedPackage->None,
WindowToolbars->{},
InitializationCellEvaluation->True,
CellGrouping->Automatic,
WindowSize->{876, 724},
WindowMargins->{{32, Automatic}, {Automatic, 0}},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellBracket->False,
ShowCellLabel->True,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
CharacterEncoding->"MacintoshAutomaticEncoding",
StyleDefinitions -> FrontEnd`FileName[{ParentDirectory[ParentDirectory[]], \
"IEC-Setup"}, "EvolvicaStyle.nb", CharacterEncoding -> "MacintoshRoman"]
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "Weighted templates"->{
    Cell[2004, 62, 379, 17, 236, "InputDemo",
      CellTags->"Weighted templates"],
    Cell[21530, 1001, 409, 18, 251, "Input",
      CellTags->"Weighted templates"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"Weighted templates", 39039, 1744}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 109, 2, 91, "Title",
  Evaluatable->False],
Cell[1888, 57, 113, 3, 62, "Subsubtitle"],
Cell[2004, 62, 379, 17, 236, "InputDemo",
  CellTags->"Weighted templates"],

Cell[CellGroupData[{
Cell[2408, 83, 80, 2, 76, "Subtitle",
  Evaluatable->False],
Cell[2491, 87, 217, 6, 44, "Text"],
Cell[2711, 95, 428, 15, 44, "Text"],
Cell[3142, 112, 286, 5, 62, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3465, 122, 76, 2, 76, "Subtitle",
  Evaluatable->False],
Cell[3544, 126, 242, 8, 26, "Text"],

Cell[CellGroupData[{
Cell[3811, 138, 108, 2, 56, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[3944, 144, 82, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[4029, 148, 155, 12, 176, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4221, 165, 80, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[4326, 171, 129, 4, 56, "Input"],
Cell[4458, 177, 758, 32, 114, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[5277, 216, 108, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[5410, 222, 82, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[5495, 226, 177, 14, 206, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5709, 245, 80, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[5814, 251, 117, 4, 56, "Input"],
Cell[5934, 257, 1344, 59, 194, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7315, 321, 114, 4, 41, "Input"],
Cell[7432, 327, 257, 5, 23, "Print"],
Cell[7692, 334, 79, 1, 40, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7808, 340, 114, 4, 41, "Input"],
Cell[7925, 346, 53, 1, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[8039, 354, 98, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[8162, 360, 86, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[8251, 364, 175, 14, 206, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[8463, 383, 84, 2, 34, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[8572, 389, 172, 5, 71, "Input"],
Cell[8747, 396, 550, 22, 82, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[9346, 424, 86, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[9435, 428, 176, 14, 206, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[9648, 447, 84, 2, 34, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[9757, 453, 172, 5, 71, "Input"],
Cell[9932, 460, 550, 23, 82, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[10543, 490, 102, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[10670, 496, 82, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[10755, 500, 206, 14, 206, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[10998, 519, 80, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[11103, 525, 191, 6, 86, "Input"],
Cell[11297, 533, 56, 1, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[11414, 541, 91, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[11530, 547, 82, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[11615, 551, 175, 14, 206, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11827, 570, 80, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[11932, 576, 173, 5, 71, "Input"],
Cell[12108, 583, 2446, 96, 194, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[14615, 686, 100, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[14740, 692, 82, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[14825, 696, 239, 15, 221, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[15101, 716, 80, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[15206, 722, 172, 5, 71, "Input"],
Cell[15381, 729, 896, 32, 114, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[16338, 768, 96, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[16459, 774, 95, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[16557, 778, 214, 14, 206, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16808, 797, 80, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[16913, 803, 201, 5, 71, "Input"],
Cell[17117, 810, 999, 37, 114, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[18177, 854, 97, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[18299, 860, 115, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[18417, 864, 234, 16, 236, "Input"],
Cell[18654, 882, 132, 15, 221, "Input"],

Cell[CellGroupData[{
Cell[18811, 901, 217, 6, 86, "Input"],
Cell[19031, 909, 2332, 82, 194, "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[21412, 997, 115, 2, 34, "Subsubsection",
  Evaluatable->False],
Cell[21530, 1001, 409, 18, 251, "Input",
  CellTags->"Weighted templates"],

Cell[CellGroupData[{
Cell[21964, 1023, 61, 1, 41, "Input"],
Cell[22028, 1026, 235, 3, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22300, 1034, 203, 6, 86, "Input"],
Cell[22506, 1042, 2145, 81, 194, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[24712, 1130, 89, 2, 40, "Subsection",
  Evaluatable->False],
Cell[24804, 1134, 208, 4, 44, "Text"],

Cell[CellGroupData[{
Cell[25037, 1142, 106, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[25146, 1146, 654, 33, 491, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[25837, 1184, 80, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[25942, 1190, 239, 7, 101, "Input"],
Cell[26184, 1199, 266, 5, 72, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[26523, 1212, 82, 2, 76, "Subtitle",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[26630, 1218, 89, 2, 78, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[26744, 1224, 101, 2, 56, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[26870, 1230, 446, 19, 266, "Input",
  InitializationCell->True],
Cell[27319, 1251, 206, 3, 70, "Message"]
}, Open  ]],
Cell[27540, 1257, 126, 2, 41, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[27691, 1263, 87, 1, 41, "Input"],
Cell[27781, 1266, 230, 4, 23, "Print"],
Cell[28014, 1272, 37, 1, 40, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[28100, 1279, 105, 2, 56, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[28230, 1285, 643, 25, 356, "Input",
  InitializationCell->True],
Cell[28876, 1312, 183, 3, 70, "Message"],
Cell[29062, 1317, 182, 3, 70, "Message"],
Cell[29247, 1322, 188, 3, 70, "Message"],
Cell[29438, 1327, 167, 3, 70, "Message"]
}, Open  ]],
Cell[29620, 1333, 183, 8, 101, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[29840, 1346, 100, 2, 56, "Subsection",
  Evaluatable->False],
Cell[29943, 1350, 63, 1, 41, "Input"],

Cell[CellGroupData[{
Cell[30031, 1355, 101, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[30135, 1359, 484, 19, 266, "Input",
  InitializationCell->True],
Cell[30622, 1380, 497, 19, 266, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[31156, 1404, 94, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[31253, 1408, 328, 14, 191, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[31618, 1427, 82, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[31703, 1431, 319, 14, 191, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[32059, 1450, 90, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[32152, 1454, 315, 14, 191, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[32504, 1473, 114, 2, 60, "Subsubsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[32643, 1479, 367, 13, 176, "Input",
  InitializationCell->True],
Cell[33013, 1494, 198, 3, 70, "Message"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[33260, 1503, 117, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[33380, 1507, 216, 6, 71, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[33633, 1518, 92, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[33728, 1522, 113, 2, 41, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[33878, 1529, 80, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[33961, 1533, 130, 7, 86, "Input",
  InitializationCell->True]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[34152, 1547, 84, 2, 62, "Section",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[34261, 1553, 145, 5, 56, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[34431, 1562, 89, 2, 60, "Subsubsection",
  Evaluatable->False],
Cell[34523, 1566, 125, 2, 41, "Input",
  InitializationCell->True],
Cell[34651, 1570, 93, 2, 41, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[34781, 1577, 103, 2, 34, "Subsubsection",
  Evaluatable->False],
Cell[34887, 1581, 651, 21, 296, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[35575, 1607, 104, 2, 34, "Subsubsection",
  Evaluatable->False],
Cell[35682, 1611, 353, 15, 206, "Input",
  InitializationCell->True],
Cell[36038, 1628, 103, 2, 41, "Input",
  InitializationCell->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[36190, 1636, 111, 2, 40, "Subsection",
  Evaluatable->False],

Cell[CellGroupData[{
Cell[36326, 1642, 393, 15, 206, "Input",
  InitializationCell->True],
Cell[36722, 1659, 185, 3, 70, "Message"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[36956, 1668, 109, 2, 40, "Subsection",
  Evaluatable->False],
Cell[37068, 1672, 355, 15, 206, "Input",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[37448, 1691, 63, 1, 41, "Input"],
Cell[37514, 1694, 39, 1, 40, "Output"]
}, Open  ]]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[37626, 1703, 78, 2, 50, "Subtitle",
  Evaluatable->False],
Cell[37707, 1707, 204, 5, 62, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

